==> mongodb_6.1.1.orig.tar.gz: mongo-r6.1.1/jstests/aggregation/sources/lookup/lookup_absorb_match.js <==
/**
 * Tests that $match stages with a variety of expressions still return the correct results if they
 * have been absorbed by $lookup stages.
 */
(function() {
"use strict";

let testDB = db.getSiblingDB("lookup_absorb_match");
testDB.dropDatabase();

let locations = testDB.getCollection("locations");
assert.commandWorked(locations.insert({
    _id: "doghouse",
    coordinates: [25.0, 60.0],
    extra: {breeds: ["terrier", "dachshund", "bulldog"]}
}));
assert.commandWorked(locations.insert({
    _id: "bullpen",
    coordinates: [-25.0, -60.0],
    extra: {breeds: "Scottish Highland", feeling: "bullish"}
}));

let animals = testDB.getCollection("animals");
assert.commandWorked(animals.insert({_id: "dog", locationId: "doghouse"}));
assert.commandWorked(animals.insert({_id: "bull", locationId: "bullpen"}));

// Test that a $match with $geoWithin works properly when performed directly on an absorbed
// lookup field.
let result = testDB.animals
                     .aggregate([
                         {
                             $lookup: {
                                 from: "locations",
                                 localField: "locationId",
                                 foreignField: "_id",
                                 as: "location"
                             }
                         },
                         {$unwind: "$location"},
                         {
                             $match: {
                                 "location.coordinates": {
                                     $geoWithin: {
                                         $geometry: {
                                             type: "MultiPolygon",
                                             coordinates: [[[
                                                 [20.0, 70.0],
                                                 [30.0, 70.0],
                                                 [30.0, 50.0],
                                                 [20.0, 50.0],
                                                 [20.0, 70.0]
                                             ]]]
                                         }
                                     }
                                 }
                             }
                         },
                         {$project: {"location.extra": false}}
                     ])
                     .toArray();
let expected =
    [{_id: "dog", locationId: "doghouse", location: {_id: "doghouse", coordinates: [25.0, 60.0]}}];
assert.eq(result, expected);

// Test that a $match with $geoIntersects works as expected when absorbed by a $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             "location.coordinates": {
                                 $geoIntersects: {
                                     $geometry: {
                                         type: "MultiPolygon",
                                         coordinates: [[[
                                             [-20.0, -70.0],
                                             [-30.0, -70.0],
                                             [-30.0, -50.0],
                                             [-20.0, -50.0],
                                             [-20.0, -70.0]
                                         ]]]
                                     }
                                 }
                             }
                         }
                     },
                     {$project: {"location.extra": false}}
                 ])
                 .toArray();
expected =
    [{_id: "bull", locationId: "bullpen", location: {_id: "bullpen", coordinates: [-25.0, -60.0]}}];
assert.eq(result, expected);

// Test that a $match with $type works as expected when absorbed by a $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             "location.extra.breeds": {
                                 $type: "array"
                             }
                         }
                     },
                     {$project: {"location.extra": false}}
                 ])
                 .toArray();
expected =
    [{_id: "dog", locationId: "doghouse", location: {_id: "doghouse", coordinates: [25.0, 60.0]}}];
assert.eq(result, expected);

// Test that a $match with $jsonSchema works as expected although ineligable for absorbtion by a
// $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             $jsonSchema: {
                                 properties: {location: {
                                    properties: {extra: {
                                        properties: {breeds: {type: "string"}}
                                    }}
                                 }}
                             }
                         }
                     },
                     {$project: {"location.extra": false}}
                 ])
                 .toArray();
expected =
    [{_id: "bull", locationId: "bullpen", location: {_id: "bullpen", coordinates: [-25.0, -60.0]}}];
assert.eq(result, expected);

// Test that a more complex $match with $jsonSchema works as expected although ineligable for
// absorbtion by a $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             $jsonSchema: {
                                 properties: {location: {properties: {extra: {minProperties: 2}}}}
                             }
                         }
                     },
                     {$project: {"location.extra": false}}
                 ])
                 .toArray();
expected =
    [{_id: "bull", locationId: "bullpen", location: {_id: "bullpen", coordinates: [-25.0, -60.0]}}];
assert.eq(result, expected);

// Test that a $match with $alwaysTrue works as expected although ineligable for absorbtion by a
// $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {$alwaysTrue: 1}
                     },
                     {$project: {"location.extra": false}},
                     {$sort: {_id: 1}}
                 ])
                 .toArray();
expected = [
    {_id: "bull", locationId: "bullpen", location: {_id: "bullpen", coordinates: [-25.0, -60.0]}},
    {_id: "dog", locationId: "doghouse", location: {_id: "doghouse", coordinates: [25.0, 60.0]}}
];
assert.eq(result, expected);

// Test that a $match with $alwaysFalse works as expected although ineligable for absorbtion by a
// $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {$alwaysFalse: 1}
                     },
                     {$project: {"location.extra": false}},
                 ])
                 .toArray();
expected = [];
assert.eq(result, expected);

// Test that a $match with $expr works as expected although ineligable for absorbtion by a $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             $expr: {
                                 $in: [25.0, "$location.coordinates"]
                             }
                         }
                     },
                     {$project: {"location.extra": false}},
                 ])
                 .toArray();
expected =
    [{_id: "dog", locationId: "doghouse", location: {_id: "doghouse", coordinates: [25.0, 60.0]}}];
assert.eq(result, expected);
}());

==> mongodb_7.0.14.orig.tar.gz: mongo-r7.0.14/jstests/aggregation/sources/lookup/lookup_absorb_match.js <==
/**
 * Tests that $match stages with a variety of expressions still return the correct results if they
 * have been absorbed by $lookup stages.
 */
(function() {
"use strict";

let testDB = db.getSiblingDB("lookup_absorb_match");
testDB.dropDatabase();

let locations = testDB.getCollection("locations");
assert.commandWorked(locations.insert({
    _id: "doghouse",
    coordinates: [25.0, 60.0],
    extra: {breeds: ["terrier", "dachshund", "bulldog"]}
}));
assert.commandWorked(locations.insert({
    _id: "bullpen",
    coordinates: [-25.0, -60.0],
    extra: {breeds: "Scottish Highland", feeling: "bullish"}
}));

let animals = testDB.getCollection("animals");
assert.commandWorked(animals.insert({_id: "dog", locationId: "doghouse"}));
assert.commandWorked(animals.insert({_id: "bull", locationId: "bullpen"}));

// Test that a $match with $geoWithin works properly when performed directly on an absorbed
// lookup field.
let result = testDB.animals
                     .aggregate([
                         {
                             $lookup: {
                                 from: "locations",
                                 localField: "locationId",
                                 foreignField: "_id",
                                 as: "location"
                             }
                         },
                         {$unwind: "$location"},
                         {
                             $match: {
                                 "location.coordinates": {
                                     $geoWithin: {
                                         $geometry: {
                                             type: "MultiPolygon",
                                             coordinates: [[[
                                                 [20.0, 70.0],
                                                 [30.0, 70.0],
                                                 [30.0, 50.0],
                                                 [20.0, 50.0],
                                                 [20.0, 70.0]
                                             ]]]
                                         }
                                     }
                                 }
                             }
                         },
                         {$project: {"location.extra": false}}
                     ])
                     .toArray();
let expected =
    [{_id: "dog", locationId: "doghouse", location: {_id: "doghouse", coordinates: [25.0, 60.0]}}];
assert.eq(result, expected);

// Test that a $match with $geoIntersects works as expected when absorbed by a $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             "location.coordinates": {
                                 $geoIntersects: {
                                     $geometry: {
                                         type: "MultiPolygon",
                                         coordinates: [[[
                                             [-20.0, -70.0],
                                             [-30.0, -70.0],
                                             [-30.0, -50.0],
                                             [-20.0, -50.0],
                                             [-20.0, -70.0]
                                         ]]]
                                     }
                                 }
                             }
                         }
                     },
                     {$project: {"location.extra": false}}
                 ])
                 .toArray();
expected =
    [{_id: "bull", locationId: "bullpen", location: {_id: "bullpen", coordinates: [-25.0, -60.0]}}];
assert.eq(result, expected);

// Test that a $match with $type works as expected when absorbed by a $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             "location.extra.breeds": {
                                 $type: "array"
                             }
                         }
                     },
                     {$project: {"location.extra": false}}
                 ])
                 .toArray();
expected =
    [{_id: "dog", locationId: "doghouse", location: {_id: "doghouse", coordinates: [25.0, 60.0]}}];
assert.eq(result, expected);

// Test that a $match with $jsonSchema works as expected although ineligible for absorbtion by a
// $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             $jsonSchema: {
                                 properties: {location: {
                                    properties: {extra: {
                                        properties: {breeds: {type: "string"}}
                                    }}
                                 }}
                             }
                         }
                     },
                     {$project: {"location.extra": false}}
                 ])
                 .toArray();
expected =
    [{_id: "bull", locationId: "bullpen", location: {_id: "bullpen", coordinates: [-25.0, -60.0]}}];
assert.eq(result, expected);

// Test that a more complex $match with $jsonSchema works as expected although ineligible for
// absorbtion by a $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             $jsonSchema: {
                                 properties: {location: {properties: {extra: {minProperties: 2}}}}
                             }
                         }
                     },
                     {$project: {"location.extra": false}}
                 ])
                 .toArray();
expected =
    [{_id: "bull", locationId: "bullpen", location: {_id: "bullpen", coordinates: [-25.0, -60.0]}}];
assert.eq(result, expected);

// Test that $match with a $jsonSchema property that will internally translate to a match
// expression node that has a path that is prefixed by the 'as' field in the lookup and that has
// children that can operate on that path (in this case, $_internalSchemaAllElemMatchFromIndex)
// works as expected although ineligible for absorbtion by a $lookup. Note that the jsonSchema below
// ensures that all elements of 'location.coordinates' are above 0 since the 'items' field is an
// object.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             $jsonSchema: {
                                 properties: {"location.coordinates": {items: {minimum: 0}}}
                             }
                         }
                     },
                     {$project: {"location.extra": false, "colors": false}}
                 ])
                 .toArray();

expected =
    [{_id: "dog", locationId: "doghouse", location: {_id: "doghouse", coordinates: [25.0, 60.0]}}];
assert.eq(result, expected);

// Test that $match with a $jsonSchema property that will internally translate to a match
// expression node that has a path that is prefixed by the 'as' field in the lookup and that has
// children that can operate on that path (in this case, $_internalSchemaMatchArrayIndex) works as
// expected although ineligible for absorbtion by a $lookup. Note that the jsonSchema below ensures
// that the first element of 'location.coordinates' is above 0 since the 'items' field is an array.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             $jsonSchema: {
                                 properties: {"location.coordinates": {items: [{minimum: 0}]}}
                             }
                         }
                     },
                     {$project: {"location.extra": false, "colors": false}}
                 ])
                 .toArray();

expected =
    [{_id: "dog", locationId: "doghouse", location: {_id: "doghouse", coordinates: [25.0, 60.0]}}];
assert.eq(result, expected);

// Test that $match with a $jsonSchema property that will internally translate to a match
// expression node that has a path that is prefixed by the 'as' field in the lookup and that has
// children that can operate that path (in this case, $_internalSchemaObjectMatch) works as expected
// although ineligible for absorbtion by a $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             $jsonSchema: {
                                 properties: {"location.extra": {type: 'object', properties: {"breeds": {type: 'string'}}}}
                             }
                         }
                     },
                     {$project: {"location.extra": false, "colors": false}}
                 ])
                 .toArray();
expected =
    [{_id: "bull", locationId: "bullpen", location: {_id: "bullpen", coordinates: [-25.0, -60.0]}}];
assert.eq(result, expected);

// Test that a $match with $alwaysTrue works as expected although ineligible for absorbtion
// by a $lookup. The $sort is to guarantee records are returned in the expected order.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {$alwaysTrue: 1}
                     },
                     {$project: {"location.extra": false}},
                     {$sort: {_id: 1}}
                 ])
                 .toArray();
expected = [
    {_id: "bull", locationId: "bullpen", location: {_id: "bullpen", coordinates: [-25.0, -60.0]}},
    {_id: "dog", locationId: "doghouse", location: {_id: "doghouse", coordinates: [25.0, 60.0]}}
];
assert.eq(result, expected);

// Test that a $match with $alwaysFalse works as expected although ineligible for absorbtion by a
// $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {$alwaysFalse: 1}
                     },
                     {$project: {"location.extra": false}},
                 ])
                 .toArray();
expected = [];
assert.eq(result, expected);

// Test that a $match with $expr works as expected although ineligible for absorbtion by a $lookup.
result = testDB.animals
                 .aggregate([
                     {
                         $lookup: {
                             from: "locations",
                             localField: "locationId",
                             foreignField: "_id",
                             as: "location"
                         }
                     },
                     {$unwind: "$location"},
                     {
                         $match: {
                             $expr: {
                                 $in: [25.0, "$location.coordinates"]
                             }
                         }
                     },
                     {$project: {"location.extra": false}},
                 ])
                 .toArray();
expected =
    [{_id: "dog", locationId: "doghouse", location: {_id: "doghouse", coordinates: [25.0, 60.0]}}];
assert.eq(result, expected);
}());
